
;+

function clicklimb,image,x,y,r,guess,error=error,radius_fix=radius_fix

;NAME:
;     CLICKLIMB
;PURPOSE:
;     Fit a circle to points clicked on with mouse
;CATEGORY:
;CALLING SEQUENCE:
;     result = clicklimb(image,x,y,r,guess,error=error)
;INPUTS:
;     image = image to be displayed
;OPTIONAL INPUT PARAMETERS:
;     guess = [x,y,r] initial guess 
;KEYWORD PARAMETERS
;     /radius_fix = The radius exact radius.  If this is set, the radius
;                   will not be fit.
;OUTPUTS:
;     x,y,r = x position of center, y position of center, radius of cicle in
;             device units. Optional.
;     result = [x,y,r] in device units.  result = -1 in case of error.
;     error = error in x,y,r (device) units
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;     Click the left mouse button on at least three points on the circle 
;     displayed (image).  Click the right mousr button when finished.
;MODIFICATION HISTORY:
;     T. Metcalf & K. Shibasaki, 1994-10-18
;     2001-June-26 TRM Added radius_fix keyword
;-

   if keyword_set(guess) then begin
      if n_elements(guess) NE 3 then message,/info,'Guess must have three elements [x,y,r].  Ignoring guess'
   endif

   nx = n_elements(image(*,0))
   ny = n_elements(image(0,*))
   if !d.window LT 0 OR !d.x_size NE nx OR !d.y_size NE ny then $
      window,/free,xs=nx,ys=ny
   wshow,!d.window

   tv,image
   x0 = -1
   y0 = -1
   REPEAT begin
      cursor,x,y,3,/device
      if !err EQ 1 then begin
         x0 = [x0,x]
         y0 = [y0,y]
         print,x,y
         plots,x,y,psym=1,/device
      endif
   endrep UNTIL !err GT 1
   nd = n_elements(x0)-1L

   if nd GE 3 then begin

      if n_elements(guess) NE 3 then begin

         ; For the initial guess, pick the three points which are furthest
         ; from each other (first point clicked is always used).  Then find
         ; the perpendicular bisector of the lines connecting these points
         ; and take the initial center as the intersection of these lines.
         ; The initial radius is the average distance of the center from the
         ; three selected points.

         ; dist. from 1st point
         d1 = sqrt((x0(1:*)-x0(1))^2+(y0(1:*)-y0(1))^2)  ; dist. from 1st point
         md1 = max(d1,mxp1)
         ; dist from 1st and 2nd points
         d12 = d1 + sqrt((x0(1:*)-x0(mxp1+1))^2+(y0(1:*)-y0(mxp1+1))^2)
         md12 = max(d12,mxp12)
         ; intermediate values
         x1 = (x0(1:*))(0)     & y1 = (y0(1:*))(0)
         x2 = (x0(1:*))(mxp1)  & y2 = (y0(1:*))(mxp1)
         x3 = (x0(1:*))(mxp12) & y3 = (y0(1:*))(mxp12)
         if y2 EQ y1 then begin
            ; Swap 3 and 1
            t = x1 & x1 = x3 & x3 = t
            t = y1 & y1 = y3 & y3 = t
         endif
         if y3 EQ y1 then begin
            ; Swap 2 and 1
            t = x1 & x1 = x2 & x2 = t
            t = y1 & y1 = y2 & y2 = t
         endif
         if (y2 EQ y1) OR (y3 EQ y1) then $
            message,'Error computing initial guess.  Reorder circle points.'
         mx1 = (x1+x2)/2.
         my1 = (y1+y2)/2.
         mx2 = (x1+x3)/2.
         my2 = (y1+y3)/2.
         x01 = float(x2-x1)
         y01 = float(y2-y1)
         x02 = float(x3-x1)
         y02 = float(y3-y1)
         ; compute intersection of perpendicular bisectors
         x = ((my1-my2) + (mx1*x01/y01 - mx2*x02/y02))/(x01/y01-x02/y02)
         y = my1 - (x-mx1)*x01/y01
         ; compute mean radius
         r = (sqrt((x1-x)^2 + (y1-y)^2) + $
              sqrt((x2-x)^2 + (y2-y)^2) + $
              sqrt((x3-x)^2 + (y3-y)^2) )/3.0
         guess = [x,y,r]
      endif

      ; fit the circle

      if keyword_set(radius_fix) then begin
         if abs(radius_fix - guess[2])/guess[2] GT 0.10 then $
            message,/info,'Warning: the radius you passed in does not seem to be correct.  Proceeding anyway using your value of the radius.'
         guess[2]=radius_fix
      endif

      fit = fit_circle(float(x0(1:*)),float(y0(1:*)),guess,radius_fix=keyword_set(radius_fix))
      x = fit(0)
      y = fit(1)
      r = fit(2)

      ; Plot the fitted circle and the slected points

      theta = findgen(360)*!dtor
      x1 = x + r*cos(theta)
      y1 = y + r*sin(theta)
      plots,x1,y1,/device

      ; compute the error = mean radius error

      error = sqrt(abs(total((x0(1:*)-x)^2 + (y0(1:*)-y)^2 - r^2)/nd))
      print,'Error: ',error

   endif else fit = -1

   return,fit

end
